#!/usr/bin/python

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

from ansible.module_utils.basic import AnsibleModule

DOCUMENTATION = '''
---
module: ovh_loadbalancer_farm_server
short_description: Manage farm server for OVH LoadBalancer
description:
    - Manage farm server for OVH (French European hosting provider) LoadBalancer.
author: "Emmanuel Cohen (@atmaniak)"
notes:
    - Uses the python OVH Api U(https://github.com/ovh/python-ovh).
      You have to create an application (a key and secret) with a consummer
      key as described into U(https://eu.api.ovh.com/g934.first_step_with_api)
requirements:
    - ovh >=  0.4.8
options:
    name:
        required: true
        description:
            - Name of the server to add
    loadbalancerid:
        required: true
        description:
            - id of OVH loadbalancer to manage
    farmid:
        required: true
        description:
            - The name of farm to manage
    address:
        required: true
        description:
            - The destination IP address to manage
    port:
        required: true
        description:
            - Destination port
    ssl:
        required: false
        description:
            - Enable or not the SSL certificate
    active:
        required: true
        choices: ['tcp', 'http', 'udp']
        description:
            - Enable or not
    state:
        required: true
        default: present
        choices: ['present', 'absent']
        description:
            - Determines wether the record is to be created/modified or deleted
    farmtype:
        required: true
        default: tcp
        choices: ['tcp', 'http', 'udp']
        description:
            - Farmtype
    endpoint:
        required: false
        default: ovh-eu
        description:
            - The endpoint to use (for instance ovh-eu)
    application_key:
        required: true
        description:
            - The applicationKey to use
    application_secret:
        required: true
        description:
            - The application secret to use
    consumer_key:
        required: true
        description:
            - The consumer key to use

'''

EXAMPLES = '''
# Add 127.0.0.1 to farm server id 1234
- ovh_loadbalancer_farm_server:
    name: dislay_name
    loadbalancerid: 1234
    farmid: 1234
    address: 127.0.0.1
    port: 443
    status: active
    farmtype: http
    ssl: yes
    state: present
    application_key: yourkey
    application_secret: yoursecret
    consumer_key: yourconsumerkey
'''

RETURN = '''
'''

try:
    import ovh
    import ovh.exceptions
    from ovh.exceptions import APIError
    HAS_OVH = True
except ImportError:
    HAS_OVH = False


def getOvhClient(ansibleModule):
    endpoint = ansibleModule.params.get('endpoint')
    application_key = ansibleModule.params.get('application_key')
    application_secret = ansibleModule.params.get('application_secret')
    consumer_key = ansibleModule.params.get('consumer_key')

    return ovh.Client(
        endpoint=endpoint,
        application_key=application_key,
        application_secret=application_secret,
        consumer_key=consumer_key
    )


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(required=True),
            loadbalancerid=dict(required=True),
            farmid=dict(required=True, type='int'),
            address=dict(required=True),
            state=dict(default='present', choices=['present', 'absent']),
            farmtype=dict(default='tcp', choices=['tcp', 'http', 'udp']),
            probe=dict(default='false'),
            weight=dict(default='0'),
            backup=dict(default='false'),
            port=dict(default='', type='int'),
            status=dict(default='active', choices=['active', 'inactive']),
            ssl=dict(type='bool', default=False),
            endpoint=dict(default='ovh-eu'),
            application_key=dict(required=True, no_log=True),
            application_secret=dict(required=True, no_log=True),
            consumer_key=dict(required=True, no_log=True),
        ),
        supports_check_mode=True
    )

    result = dict(
        changed=False
    )

    if not HAS_OVH:
        module.fail_json(msg='ovh-api python module is required to run this module ')

    # Get parameters
    name = module.params.get('name')
    loadbalancerid = module.params.get('loadbalancerid')
    farmid = module.params.get('farmid')
    farmtype = module.params.get('farmtype')
    address = module.params.get('address')
    state = module.params.get('state')
    weight = module.params.get('weight')
    ssl = module.params.get('ssl')
    port = module.params.get('port')
    status = module.params.get('status')

    # Connect to OVH API
    client = getOvhClient(module)
    # Check that the LoadBalancer & farm exists
    try:
        client.get('/ipLoadbalancing/{}/{}/farm/{}'.format(loadbalancerid, farmtype, farmid))
    except APIError as apiError:
        module.fail_json(
            msg='Unable to call OVH api for getting the object '
                'check application key, secret, consumerkey and parameters. '
                'Error returned by OVH api was : {0}'.format(apiError))

    farm_servers = client.get('/ipLoadbalancing/{}/{}/farm/{}/server'.format(loadbalancerid, farmtype, farmid))
    defined_server = dict({})
    for server in farm_servers:
        myserver = client.get('/ipLoadbalancing/{}/{}/farm/{}/server/{}'.format(loadbalancerid, farmtype, farmid, server))
        defined_server[myserver['address']] = myserver

    # module.fail_json(msg='{}'.format(type(defined_server[myserver['address']])))
    need_update = False
    already_exists = False

    if address in defined_server:
        already_exists = True
        if defined_server[address]['displayName'] != name:
            need_update = True

    if not module.check_mode and already_exists and state == "absent":
        try:
            server_to_delete = defined_server[address]["serverId"]
            print("action: delete")
            client.delete('/ipLoadbalancing/{}/{}/farm/{}/server/{}'.format(loadbalancerid, farmtype, farmid, server_to_delete))
            result['changed'] = True
        except APIError as apiError:
            module.fail_json(
                msg='Unable to call OVH api for removing the object '
                'check application key, secret, consumerkey and parameters. '
                'Error returned by OVH api was0 : {0}'.format(apiError))

    if not module.check_mode and need_update and state == "present":
        try:
            client.put('/ipLoadbalancing/{}/{}/farm/{}/server/{}'.format(loadbalancerid, farmtype, farmid, defined_server[address]["serverId"]), address=address, port=port, weight=weight, ssl=ssl, status=status, displayName=name)
            result['changed'] = True
        except APIError as apiError:
            module.fail_json(
                msg='Unable to call OVH api for updating the object '
                'check application key, secret, consumerkey and parameters. '
                'Error returned by OVH api was1 : {0}'.format(apiError))

    if not module.check_mode and not need_update and not already_exists and state == "present":
        try:
            client.post('/ipLoadbalancing/{}/{}/farm/{}/server'.format(loadbalancerid, farmtype, farmid), address=address, port=port, weight=weight, ssl=ssl, status=status, displayName=name)
            result['changed'] = True
        except APIError as apiError:
            module.fail_json(
                msg='Unable to call OVH api for creating the object '
                'check application key, secret, consumerkey and parameters. '
                'Error returned by OVH api was2 : {0}{1}'.format(apiError, need_update))

    module.exit_json(**result)


if __name__ == '__main__':
    main()
